/*
LockDis.  A very simple X screen locker.  No flying toasters.
Copyright (C) 2009  Brett McNamara <brett@chaingang.org>

This file is part of LockDis.

LockDis is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Foobar is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <X11/Xlib.h>
#include <X11/keysym.h>

#include <stdbool.h>

#include "lockdis.h"
#include "passinfo.h"


typedef struct StateStruct {
	DisplayInfo di;
	XEvent ev;
	char password[PASSWORD_MAX];
	int passwordSize;
	//char *hashToMatch;
} State;

/*
bool isPasswordMatch(State *st) {
	return (strcmp(getHashForPassword(st->password), st->hashToMatch)==0);
}
*/

int errorHandler(Display *display, XErrorEvent *error) {
	fprintf(stderr, "%s: X Error %d\n", APP_NAME, error->error_code);

	return 0;
}




void processKeyPress(State *st) {
	char buffer[1];
	KeySym keysym;

	XLookupString((XKeyEvent *) &st->ev, buffer, sizeof buffer, &keysym, 0);
	
	if (keysym == XK_Return) {
		st->password[st->passwordSize] = 0;
		if (isPasswordMatch(st->password)) {
			XUngrabKeyboard(st->di.display, CurrentTime);
			exit(EXIT_SUCCESS);
		} else {
			st->passwordSize = 0;
		}
		
	} else if (keysym == XK_BackSpace) {
		if (st->passwordSize > 0) { st->passwordSize--; }
		
	} else if (keysym == XK_Escape) {
		st->passwordSize = 0;
		
	} else if (isprint(*buffer) && st->passwordSize < PASSWORD_MAX) {
		st->password[st->passwordSize++] = *buffer;
	}
	
	showScreen(&st->di, st->passwordSize);
}


bool processEvent(State *st) {
	
	if (st->ev.type==Expose) {
		if (st->ev.xexpose.count == 0 && *st->di.windows == st->ev.xexpose.window) {
			showScreen(&st->di, st->passwordSize);
		}
		return true;
		
	} else if (st->ev.type==KeyPress) {
		processKeyPress(st);
		return true;
		
	} else if (st->ev.type==KeyRelease) {
		return true;
		
	} else if (st->ev.type==MappingNotify) {
		XRefreshKeyboardMapping((XMappingEvent *) &st->ev);
		return true;
		
	}
	return false;
}


void init(State *st) {
	st->passwordSize = 0;
	//st->hashToMatch = getUserHash();
	initDisplay(&st->di);
	XSetErrorHandler(errorHandler);
}


void process(State *st) {
	XNextEvent(st->di.display, &st->ev);
	if (!processEvent(st)) {
		raiseWindows(&st->di);
	}
}


void getNewPassInteractive() {
	char *p, buff[PASSWORD_MAX];
	
	p = getpass("Enter new password: ");
	if (strlen(p)>=PASSWORD_MAX) {
		fprintf(stderr, "That a really long password, please keep it under %d characters.\n", PASSWORD_MAX);
		exit(EXIT_FAILURE);
	}
	
	strcpy(buff, p);
	
	p = getpass("Re-enter password: ");
	if (strcmp(buff,p)!=0) {
		fprintf(stderr, "Passwords do not match\n");
		exit(EXIT_FAILURE);
	}
	
	writePassword(p);
}




void usage() {
	printf("usage:\n");
	printf("   %s\n", APP_NAME);
	printf("   Normal launch, screen will be locked\n\n");
	printf("   %s -p\n", APP_NAME);
	printf("   Initialize password. Must be done before program can be used.\n\n");
	printf("   %s -h\n", APP_NAME);
	printf("   You are here\n\n");
	exit(EXIT_FAILURE);
}


int main(int argc, char *argv[]) {
	if (argc==1) {
		State st;
		init(&st);
		for (;;) { process(&st); }
	} else if (argv[1][0]!='-') {
		usage();
	} else if (argv[1][1]=='p' && argc==2) {
		getNewPassInteractive();
	} else {
		usage();
	}
	return EXIT_SUCCESS;
}
