/*
LockDis.  A very simple X screen locker.  No flying toasters.
Copyright (C) 2009  Brett McNamara <brett@chaingang.org>

This file is part of LockDis.

LockDis is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Foobar is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <stdio.h>
#include <pwd.h>
#include <sys/types.h>
#include <string.h>

#include <stdbool.h>
#include <unistd.h>

#include "sha512crypt.h"
#include "passinfo.h"

#define SALT "$6$lock.dis.rules2012"
// #define HASH_SIZE getNeededHashSize(SALT)
#define HASH_SIZE 132
#define PATH_SIZE 1024

static const char *getPasswordHashLocation() {
	static char fileName[PATH_SIZE]={0};
	struct passwd *pw = NULL;
	
	if (fileName[0]==0) {
		pw = getpwuid(getuid());
		if (pw == NULL || sprintf(fileName, "%s/.lockdis", pw->pw_dir)==0) {
			fprintf(stderr, "Can't determine file location.\n");
			exit(EXIT_FAILURE);
		}
	}
	return fileName;
}



static const char *getUserHash() {
	static char userHash[HASH_SIZE] = {0};
	
	if (userHash[0]==0) {
		FILE *fp;
		int hashSize;
		const char *fileName = getPasswordHashLocation();
		
		if ((fp = fopen(fileName, "r"))==NULL) {
			fprintf(stderr, "Can't open output file %s!\n", fileName);
			exit(EXIT_FAILURE);
		}
		
		fscanf(fp, "%s\n", userHash);
		fclose(fp);
	}
	
	return userHash;
}


static char *getHashForPassword(const char *password) {
	return sha512_crypt(password, SALT);
}

bool hasPassword() { return getUserHash()!=NULL; }

bool isPasswordMatch(const char *password) {
	return (strcmp(getHashForPassword(password), getUserHash())==0);
}

void writePassword(const char *password) {
	FILE *fp;
	const char *fileName = getPasswordHashLocation();
	
	if ((fp = fopen(fileName, "w"))==NULL) {
		fprintf(stderr, "Can't open output file %s!\n", fileName);
		exit(EXIT_FAILURE);
	}
	fprintf(fp, "%s\n", getHashForPassword(password));
	fclose(fp);
}
